require(rhdf5)
# # # # # .FILE_INFO.valid <- function(object){return (TRUE)}
setClass ( 
  Class = "FILE_INFO" , 
  representation =representation (
    fid="H5IdComponent" , fname="character"
  ),
  prototype = prototype (
    fid = NULL, fname = ""),
    
  validity = function ( object ){
    cat ( "------- FILE_INFO : method inspector --------  \n " )
    if(is( object@fid, "H5IdComponent") == FALSE ){
      stop ( " [ FILE_INFO : method inspector] Error in H5 file name")
      }else{}
    return(TRUE)
  }
)


file_info <- function(fname){
  # Create a FILE_INFO object 
  cat("------- FILE_INFO : method constructor --------\n")
  if (missing(fname))
    stop("Missing file name")
  new(Class="FILE_INFO", filename = fname)
  }

setMethod(
  f = "initialize",
  signature = "FILE_INFO",
  definition = function(.Object, filename){
    cat("------- FILE_INFO : method initializator  --------\n")
    .Object@fname <- filename
    tmp <- H5Fopen(filename)
    .Object@fid <- tmp  
    validObject (.Object)
    return(.Object)
  }
) 


setMethod(
  # Get an object attribute
  f = "[",
  signature = "FILE_INFO",
  definition = function(x, i, j, drop){
    cat("------- FILE_INFO : method get  --------\n")
    switch (EXPR = i,
      "fid" = {return(x@fid)},
      "fname" = {return(x@fname)},
      stop("Unknown class attribute")
    )
  }
)  


setReplaceMethod(
  # Modify an object attribute
  f = "[",
  signature = "FILE_INFO",
  definition = function(x , i , j, value){
    cat("------- FILE_INFO : method set  --------\n")
    switch (EXPR = i,
      "fid" = {x@fid <- value},
      "fname" = {x@fname <- value},
      stop("Unknown class attribute")
    )
    validObject (x)
    return(x)    
  } 
) 
  

setGeneric(
  # Display the contents of main groups
  name = "displayContents",
  def = function(object){
    standardGeneric("displayContents")}
)
setMethod(f = "displayContents", 
	  signature = "FILE_INFO",
	  definition = function (object) {
	    cat ( "-------- FILE_INFO : method displayContents  --------\n " )
	    return(h5ls(object@fname, FALSE))   
	    cat ( "-------- End method displayContents  --------\n " )
	  }
)

          
setGeneric(
  # Display main groups and sub groups
  name = "listGroups",
  def = function(object){
    standardGeneric("listGroups")}
)
setMethod(f = "listGroups", "FILE_INFO",
          definition = function (object) {
	    cat ( "-------- FILE_INFO : method listGroups  --------\n " )
	    L = list()
	    br = h5ls(object@fname)
	    
	    for (i in 1:dim(br)[1]){	
	      if (br$otype[i] == "H5I_GROUP"){
		if (br$group[i] == "/")
		  L[length(L) + 1] <- paste0(br$group[i], br$name[i])
		else
		  L[length(L) + 1] <- paste0(br$group[i],"/", br$name[i])
	      }
	    }
	    return(L)
	    cat ( "-------- End method listGroups  --------\n " )
          }
) 
         
         
setGeneric(
  # Return TRUE if the group exists, else return FALSE
  name = "isGroup",
  def = function(object,grname){
    standardGeneric("isGroup")}
)
 setMethod(f = "isGroup", 
	   signature = "FILE_INFO",
           definition = function (object,grname) {
	    cat ( "-------- FILE_INFO : method isGroup  --------\n ")
	    lst = listGroups(object)
  
	    if (is.na(match(grname, lst)))
	      return(FALSE)
	    else
	      return(TRUE)
	    cat ( "-------- End method isGroup  --------\n " )  
          }
)          


setGeneric(
  # Get the number of file attributes
  name = "getNFileAttr",
  def = function(object){
    standardGeneric("getNFileAttr")}
)
setMethod(f = "getNFileAttr", "FILE_INFO",
          definition = function (object) {
	    cat ( "-------- FILE_INFO : method getNFileAttr   --------\n " )
	    return(H5Oget_num_attrs(object@fid))    
	    cat ( "-------- End method getNFileAttr   --------\n " )
          }
)


setGeneric(
  # Get a file attribute
  name = "getFileAttr",
  def = function(object, attr){
    standardGeneric("getFileAttr")}
)
setMethod(f = "getFileAttr", "FILE_INFO",
          definition = function (object, attr) {
	    cat ( "-------- FILE_INFO : method getFileAttr  --------\n " ) 
	    id <- H5Aopen( object@fid, attr)
 	    value <- H5Aread(id)
	    return(value)
	    H5Aclose(id)
	    cat ( "-------- End method  getFileAttr  --------\n " )
          }
)


setGeneric(
  # Create a group from GROUP_INFO constructor
  name = "getGroup",
  def = function(FILE_INFO, grname){
    standardGeneric("getGroup")}
)
setMethod(f = "getGroup", "FILE_INFO",
          definition = function (FILE_INFO, grname) {
	    cat ( "-------- FILE_INFO : method getGroup  --------\n " ) 
	    newGroup = group_info(FILE_INFO@fid, grname)
	    cat ( "-------- End method  getGroup  --------\n " )
	    return(newGroup)
	    
          }
)
